'''    Name: QFolderTree (.inc)
'''  Author: David Burkley
'''   Email: burkleyd@yahoo.com
'''    Date: Sept. 29th, 2009
'''
''' You can "not" distribute this source code
''' without asking and receiving verifiable
''' permission from myself. If I do give
''' permission... this entire header must
''' remain intacted with this source code.
'''
''' You may "freely" use this source code for
''' "non-profitable" programs only.
'''
''' For programs in which you'll profit from,
''' it "can not" be used until a verifiable
''' monetary compensation has been agreed upon.
'''
''' If you do make use of this source code in
''' your "non-profitable" program... all I ask
''' in return is... please give credit, where
''' credit is due. You can do this by simply
''' mentioning to your end-users (perhaps in
''' an "About" menu option), my name, my email
''' address, and my website.

'' I apologize if there's insufficient comments.
'' If you need specific information about the
'' code below... please contact me via email
'' with the word "RapidQ" in the subject line.
'' (emails without it, will be deleted automatically)

' Sept. 23rd, 2009
' This is my endeavor at creating an alternative to QDirTree.
' It doesn't have all the features of QDirTree but if all you
' need is a way of letting someone select a drive or folder
' and maybe display files based on the selected drive or folder...
' then this should work quite well for you. I think I was able
' to keep the memory usage to a minimum. And it can be edited
' to better suit your needs. But if you're going to edit it...
' make a backup copy first. It took me a very long time to
' figure the mechanics of a QTreeView, in order to turn it into
' something similar to a QDirTree. Maybe some of my coding below
' could've have been done in a better way. But heck...it works.
' If after experimenting with the code below... you find that
' changes could be made to improve the speed... PLEASE let me
' know and I'll give you credit as an official co-author.
'
' Depending on how you add this $Include file to your source code...
' you might have to rename some items in "this" file to avoid Naming
' Conflicts. If you do... I'd suggest adding "FL_" to the beginning
' of items in this file that conflict with your items. BUT... make
' sure you change ALL instances in this file of the same item.
' If you miss just one... this coding might not work!
'
' Sept. 24th, 2009
' Took out the Dir$ command and added a QFileListBox to retrieve
' folder names. This cut the time spent executing the GetRealSubFolders
' subroutine in half. The only drawback to doing this way is... it
' forces the programmer to add one line of coding to set the parent
' property of the QFileListBox. I tried setting it from within this
' $Include file... but it doesn't work. Why? Because there's no way
' of knowing who the parent of QFolderTree is going to be.
'
' Sept. 28th, 2009
' Added a line of code to set the Duplicates property of ParsedFolders
' (a QStringList) to dupAccept. I apparently didn't take into account
' the possibility of someone setting the InitialDir to a path that
' "might" contain duplicate folder names. Without this property being
' set properly... if the InitialDir looked something like...
' "d:\rapidq\source code\rapidq\tests" the parsing would have generated
' a list that looks like... "d:", "rapidq", "source code", "tests".
' Which obviously is not right. It should look like...
' "d:", "rapidq", "source code", "rapidq", "tests"
'
' Sept. 29th, 2009
' With clues from JohnK I was able to solve a problem that I did not
' foresee. I assumed everybody has a floppy drive (typically "A:\").
' Well apparently that's not the case. John's computer only has a
' "C:\" and "D:\" drive. A variable below named ExpandedIndex was
' preset to 0 (zero) and is used to prevent the OnExpanding subroutine
' from running 2x times in a row. With the way John's computer is
' setup... his "C:\" drive being ItemIndex 0 (zero), and the
' ExpandedIndex being preset to 0 (zero)... the OnExpanding subroutine
' was being bypassed when he tried setting InitialDir to a folder on
' his "C:\" drive. The solution was... simply to preset ExpandedIndex
' to -1 in the Constructor section and also to reset it back to -1 in
' the OnCollapsing subroutine.

'You may (or may not) have to hardcode the full path name with these icon
'resources. I extracted each (16x16x8) icon from the "shell32.dll" file.
'(a progam called IcoFx does a real nice job of it AND it's freeware)
'If they're too small... you could extract larger ones from the "shell32.dll".
'(but I would not go any bigger than the 24x24 icons)
$Resource ICO1 As "fd.ico"
$Resource ICO2 As "hd.ico"
$Resource ICO3 As "net.ico"
$Resource ICO4 As "cd.ico"
$Resource ICO5 As "ram.ico"
$Resource ICO6 As "folder.ico"

'Constants used to change the style of the QTreeView
'(gets rid of that annoying ToolTip that lingers too long)
Const GWL_STYLE      = (-16)
Const TVS_NOTOOLTIPS = &H80

'Constant used to filter out files (when SafeMode = False)
Const FILE_ATTRIBUTE_DIRECTORY = &H10

'API functions used, with the constants above, to change the style of the QTreeView
Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" _
                 (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" _
                 (ByVal hWnd As Long, ByVal nIndex As Long, _
                  ByVal dwNewLong As Long) As Long

'API function used to determine the drive type
'(i.e. floppy drive, hard drive, network drive, cd-rom, ram drive)
'See comments in subroutine "SetIcon" on how to determine a folder
Declare Function GetDriveType Lib "kernel32" Alias "GetDriveTypeA" _
                 (ByVal nDrive As String) As Long

'API function used to determine all the "currently" available drive letters
'There's no WndProc to check for drives that become available after the program starts
'(drive letters are added to the QTreeView using AddItems)
Declare Function GetLogicalDrives Lib "kernel32" Alias "GetLogicalDrives" _
                 () As Long

'API function used, with the filter constant above,
'to add only folders to the QTreeView (when SafeMode = False)
'(folders are added to the QTreeView using AddSubItem)
Declare Function GetFileAttributes Lib "kernel32" Alias "GetFileAttributesA" _
                 (ByVal lpFileName As String) As Long

'API function used to avoid a system error message
'if someone has a floppy drive and no disk in it.
Declare Function GetVolumeInformation Lib "kernel32" Alias "GetVolumeInformationA" _
                 (ByVal lpRootPathName As Long, ByVal lpVolumeNameBuffer As Long, _
                  ByVal nVolumeNameSize As Long, ByVal lpVolumeSerialNumber As Long, _
                  ByVal lpMaximumComponentLength As Long, ByVal lpFileSystemFlags As Long, _
                  ByVal lpFileSystemNameBuffer As Long, ByVal nFileSystemNameSize As Long) As Long

Type QFolderTree Extends QTreeView
  PRIVATE:
   'Used to prevent the OnExpanding event from occurring 2x in a row
   '(note: this only occurs under certain rare conditions)
    ExpandedIndex As Long
    'Used to store the current directory
    CurrentDir As String
   'The ImageList index is used to associate the proper
   'icon with the appropriate drive type or folder
    IcoImageList As QImageList
    'Using a QFileListBox to retrieve the folder names and add them to a QTreeView
    'is 50% faster than doing the same thing with the Dir$ command.
    DirList As QFileListBox

  PUBLIC:
    'Used externally to determine if readonly/hidden/system/directory/archive folders will be displayed
    'SafeMode = False means YES... display almost any kind of folder
    'SafeMode = True means NO... do not display every kind of folder
    'Be aware... setting SafeMode to False can mean slower response
    'time if a folder has a very large number of subfolders
    SafeMode As Byte
    'Used externally to set the "startup" directory
    'If not set prior to startup... it defaults to...
    'the folder where the executable resides
    InitialDir As String

  PRIVATE:
    Function GetVolInfo(ByVal path As String) As Long
        Dim RetVal                 As Long
        Dim SerialNumber           As Long
        Dim MaximumComponentLength As DWord
        Dim VolumeNameBuffer       As String
        Dim FileSystemNameBuffer   As String
        Dim RootPathName           As String

        RetVal                 = 0
        SerialNumber           = 0
        MaximumComponentLength = 0
        VolumeNameBuffer       = Space$(255)
        FileSystemNameBuffer   = Space$(255)
        RootPathName           = Space$(255)

        RootPathName = path
        RetVal = GetVolumeInformation(VarPtr(RootPathName), VarPtr(VolumeNameBuffer), 255, VarPtr(SerialNumber), VarPtr(MaximumComponentLength), 0, VarPtr(FileSystemNameBuffer), 255)

        Result = RetVal

        RetVal                 = 0
        SerialNumber           = 0
        MaximumComponentLength = 0
        VolumeNameBuffer       = ""
        FileSystemNameBuffer   = ""
        RootPathName           = ""
    End Function

    Function ExtractPathName(ByVal index As Long) As String
      Dim I    As Long
      Dim L    As Long
      Dim N    As Long
      Dim Path As String

      I    = index
      L    = This.Item(I).Level
      N    = 0
      Path = This.Item(I).Text

      'Loop backwards from the ItemIndex that was passed as
      'a parameter... until the root directory is reached
      'Each time the item Level drops by a value of 1...
      'add the folder name to the variable. For example...
      '"system32"
      '"windows\system32"
      '"c:\windows\system32"
      Do
        N = This.Item(I).Level
        If N = (L - 1) Then
          Path = This.Item(I).Text & "\" & Path
          Dec(L, 1)
        End If
        Dec(I, 1)
        DoEvents
      Loop Until L = 0
      'The return value does not have a "\" on the end
      'The subroutines below will add it at the appropriate time (if need be)
      Result = Path

      I    = 0
      L    = 0
      N    = 0
      Path = ""
    End Function

    Sub SetIcon(ByVal index As Long, ByVal path As String)
      'When trying to set the icon for a folder... you need to trick GetDriveType
      ' (note: when passing a folder as the path parameter... make sure there is no "\" on the end!)
      'The GetDriveType returns a value of 1... when checking a folder (without a "\" on the end),
      'causing it fall into the [Unrecognized] category
      ' (a return value of 1 actually means... DRIVE_NO_ROOT_DIR)
      ' (a return value of 0 actually means... DRIVE_UNKNOWN)
      ' (since neither indicates a folder... lump them together in the "Else" condition)
      'If path parameter DOES have a "\" on the end... GetDriveType will return
      'the type of drive that the root of the path parameter belongs to
      ' (e.g. c:\windows\ returns 3, a [Fixed Drive])
      Select Case GetDriveType(path)
        Case 2     '[Removable]
          This.Item(index).ImageIndex = 0
          This.Item(index).SelectedIndex = 0
        Case 3     '[Fixed Drive]
          This.Item(index).ImageIndex = 1
          This.Item(index).SelectedIndex = 1
        Case 4     '[Remote]
          This.Item(index).ImageIndex = 2
          This.Item(index).SelectedIndex = 2
        Case 5     '[CD - Rom]
          This.Item(index).ImageIndex = 3
          This.Item(index).SelectedIndex = 3
        Case 6     '[Ram Disk]
          This.Item(index).ImageIndex = 4
          This.Item(index).SelectedIndex = 4
        Case Else  '[Unrecognized]
          This.Item(index).ImageIndex = 5
          This.Item(index).SelectedIndex = 5
      End Select
    End Sub

    Sub GetFakeSubFolders(ByVal index As Long, ByVal path As String)
      Dim N              As Long
      Dim RetVal         As Long
      Dim SubFolderName  As String
      Dim FullPathName   As String
      Dim PathName       As String

      N              = 0
      SubFolderName  = ""
      FullPathName   = ""
      PathName       = IIf(Right$(path,1) = "\", path, path & "\")

      RetVal = IIf(This.Item(index).Level = 0, This.GetVolInfo(PathName), GetDriveType(path))

      If RetVal Then
        'To speed up the process of determining whether or not
        'to show a "+" button next to root drive or folder
        '(indicating that it DOES have subfolders)...
        'check the path for "ANY" subfolders and if/when one is found...
        'increment a variable and exit the checking process
        SubFolderName = Dir$(PathName & "*.", FILE_ATTRIBUTE_DIRECTORY)
        FullPathName  = PathName & SubFolderName
        'Filter out symbolic folders (i.e. "." and "..")
        '(you'll see symbolic folders when using DOS)
        If Left$(SubFolderName,1) > Chr$(47) Then Inc(N, 1)
        'If a subfolder wasn't immediately found do some more checking
        If N = 0 Then
          Do
            SubFolderName = Dir$
            FullPathName  = PathName & SubFolderName
            'Filter out symbolic folders (i.e. "." and "..")
            '(you'll see symbolic folders when using DOS)
            If Left$(SubFolderName,1) > Chr$(47) Then Inc(N, 1)
            DoEvents
            'Even if N never gets to 1... SubFolderName will equal "" when
            'the Dir$ command can not find any more subfolders
            'But... if N does equal more than 0... force an immediate exit
            If N > 0 Then SubFolderName = ""
          Loop Until SubFolderName = ""
        End If

        'If ANY subfolders where found... add a "blank" child item.
        '(the blank item will be removed when it's time to show the "real" subfolders in the OnExpanding event)
        'Adding a single child (to represent that subfolders do exist)... speeds this whole process up dramatically
        If N > 0 Then This.AddChildItems(index, " ")
      End If

      N              = 0
      RetVal         = 0
      SubFolderName  = ""
      FullPathName   = ""
      PathName       = ""
    End Sub

    Sub GetRealSubFolders(ByVal index As Long, ByVal path As String)
      Dim I              As Long
      Dim ChildIndex     As Long
      Dim SubFolderName  As String
      Dim FullPathName   As String
      Dim PathName       As String
      Dim FolderList     As QStringList

      I              = 0
      ChildIndex     = 0
      SubFolderName  = ""
      FullPathName   = ""
      PathName       = IIf(Right$(path,1) = "\", path, path & "\")
      FolderList.Clear

      Select Case This.SafeMode
        Case False
          'This combination of the QFileListBox's mask and filters
          'will retrieve almost all folder names.
          This.DirList.Mask = ("*.*")
          This.DirList.AddFileTypes(ftReadOnly, ftHidden, ftSystem, ftDirectory)
          This.DirList.DelFileTypes(ftNormal)
          This.DirList.Directory = PathName
          For I = 0 To (This.DirList.ItemCount-1)
            'Need to remove the square brackets first
            SubFolderName = This.DirList.Item(I) - "[" - "]"
            DoEvents
            'Then filter out the symbolic folders
            If Left$(SubFolderName,1) > Chr$(47) Then
              FullPathName = PathName & SubFolderName
              'Because of the mask and filters (used above)...
              'there's a need to filter out any files that may be in the list
              If GetFileAttributes(FullPathName) And FILE_ATTRIBUTE_DIRECTORY Then
                This.AddChildItems(index, SubFolderName)
                ChildIndex = index + This.Item(index).Count
                This.SetIcon(ChildIndex, PathName & SubFolderName)
              End If
            End If
          Next I
        Case True
          'This combination of the QFileListBox's mask and filters
          'will retrieve only normal folder names.
          This.DirList.Mask = ("*.")
          This.DirList.AddFileTypes(ftDirectory)
          This.DirList.DelFileTypes(ftNormal)
          This.DirList.Directory = PathName
          For I = 0 To (This.DirList.ItemCount-1)
            'Need to remove the square brackets first
            SubFolderName = This.DirList.Item(I) - "[" - "]"
            'Then filter out the symbolic folders
            If Left$(SubFolderName,1) > Chr$(47) Then
              'Because of the mask and filters (used this time)...
              'there's NO need to filter out any files that may be in the list
              FullPathName = PathName & SubFolderName
              This.AddChildItems(index, SubFolderName)
              ChildIndex = index + This.Item(index).Count
              This.SetIcon(ChildIndex, PathName & SubFolderName)
            End If
          Next I
      End Select

      I              = 0
      ChildIndex     = 0
      SubFolderName  = ""
      FullPathName   = ""
      PathName       = ""
      FolderList.Clear
    End Sub

    Sub FolderTree_OnExpanding(ByVal index As Long, ByVal allowed As Long)
      Dim I As Long
      Dim N As Long

      I = 0
      N = 0

      'As mentioned above... condition this so
      'that it does not get executed 2x in a row
      'If it does... it screws the QTreeView up royally!
      If This.ExpandedIndex <> index Then
        This.Cursor = crHourGlass
        'First delete the "fake" subfolder item
        This.DelItems(index + 1)
        'Now go get the "real" subfolder names and add them to the QTreeView
        This.GetRealSubFolders(index, This.ExtractPathName(index))
        'Before the QTreeView is completely expanded...
        'check the just added "real" subfolders for any possible subfolders
        'If any are found... add a "fake" subfolder item
        '(so that the "+" button shows next to them)
        N = index + This.Item(index).Count
        For I = N To (index + 1) Step -1
          This.GetFakeSubFolders(I, This.ExtractPathName(I))
        Next I
        This.Cursor = crDefault
        'Store the index to prevent a 2nd execution
        '(note: the OnCollpasing event resets it)
        This.ExpandedIndex = index
      End If

      I = 0
      N = 0
    End Sub

    Sub FolderTree_OnCollapsing(ByVal index As Long, ByVal allowed As Long)
      Dim ExitFlag As Byte
      Dim C        As Long
      Dim I        As Long
      Dim L        As Long
      Dim N        As Long

      ExitFlag = 0
      C        = 0
      I        = 0
      L        = 0
      N        = 0

      'This process was tricky to figure
      'out so that it would work properly
      'Start at the ItemIndex that's Collapsing
      'and check upwards into the ItemIndexes
      'for any/all possible child items
      'If any are found increment a variable
      'accordingly and when the item Level
      'drops (or the ItemCount has been reached)...
      'drop out of the loop and delete the total
      'number of child items that were found (if any)
      I = index + 1
      L = This.Item(I).Level
      Do
        C = 1
        If I = (This.ItemCount-1) Then
          ExitFlag = 1
        ElseIf This.Item(I).Level >= L Then
          Inc(N, 1)
          If This.Item(I).HasChildren = True Then
            Inc(N, This.Item(I).Count)
            Inc(C, This.Item(I).Count)
          End If
        ElseIf This.Item(I).Level < L Then
          ExitFlag = 1
        End If
        Inc(I, C)
        DoEvents
      Loop Until ExitFlag = 1
      C = index + N
      For I = C To (index + 1) Step -1
        This.DelItems(I)
      Next I
      'Reset the OnExpanding conditional variable
      This.ExpandedIndex = -1
      'Rather than running the GetFakeSubFolders again...
      'and since this folder obviously "had" subfolders...
      'I simply add a "fake" subfolder item
      '(note: this isn't the best/safest thing to do... but it's the fastest)
      This.AddChildItems(index, " ")

      ExitFlag = 0
      C        = 0
      I        = 0
      L        = 0
      N        = 0
    End Sub

    Sub SetInitialDir(path As String)
      Dim I             As Long
      Dim N             As Long
      Dim PathName      As String
      Dim FolderName    As String
      Dim ParsedFolders As QStringList

      I          = 0
      N          = 0
      PathName   = IIf(Right$(path,1) = "\", UCase$(path), UCase$(path & "\"))
      FolderName = ""
      ParsedFolders.Clear
      'Setting the Duplicates property to dupAccept will
      'prevent an error from occurring "if" the InitialDir
      'has embeded folders with the same name.
      '(e.g. "d:\rapidq\source codes\rapidq\tests")
      ParsedFolders.Duplicates = dupAccept

      If DirExists(PathName) Then
        'Parsing the path parameter will result in something like...
        '"c:"
        '"windows"
        '"system32"
        ParsedFolders.Parse(PathName, "\")
        'Now find and expand each QStringList item
        '(UCase$ ensures exact matches are found)
        Do
          FolderName = UCase$(This.Item(I).Text)
          If FolderName = UCase$(ParsedFolders.Item(N)) Then
            This.ItemIndex = I
            This.Expand(I, False)
            Inc(N, 1)
          End If
          Inc(I, 1)
        Loop Until N = ParsedFolders.ItemCount
        This.ItemIndex = (I - 1)
        'Update the internal (Private) path variable so that it can be
        'retrieved externally using the (Public) Directory function below
        This.CurrentDir = PathName
      Else
        'If an invalid directory was passed...
        'set the initial directory to where the executable resides
        This.InitialDir = Command$(0) - Application.ExeName
        PathName = IIf(Right$(This.InitialDir,1) = "\", UCase$(This.InitialDir), UCase$(This.InitialDir & "\"))
        ParsedFolders.Parse(PathName, "\")
        Do
          FolderName = UCase$(This.Item(I).Text)
          If FolderName = UCase$(ParsedFolders.Item(N)) Then
            This.ItemIndex = I
            This.Expand(I, False)
            Inc(N, 1)
          End If
          Inc(I, 1)
        Loop Until N = ParsedFolders.ItemCount
        This.ItemIndex = (I - 1)
        This.CurrentDir = PathName
      End If

      I          = 0
      N          = 0
      PathName   = ""
      FolderName = ""
      ParsedFolders.Clear
    End Sub

  PUBLIC:
    'Returns the current directory when setting a variable or property
    '(e.g. MyPath$ = FolderTree.Directory or FileListBox.Directory = FolderTree.Directory)
    Function Directory() As String
      Result = This.CurrentDir
    End Function

    'If you have a need to do more than just retrieve the
    'path name in an OnClick event... add your code to this subroutine
    'Do not create an external subroutine that never calls this subroutine
    '(if you do... you won't get the current directory name)
    Sub FolderTree_OnClick
      Dim FullPathName As String

      FullPathName = ""

      If This.Item(This.ItemIndex).Level > 0 Then
        If This.Item(This.ItemIndex).Selected Then
          FullPathName = This.ExtractPathName(This.ItemIndex) & "\"
          If FullPathName <> This.CurrentDir Then
            'Update the internal (Private) path variable so that it can be
            'retrieved externally using the (Public) Directory function above
            This.CurrentDir = FullPathName
          End If
        End If
      ElseIf This.Item(This.ItemIndex).Level = 0 Then
        If This.Item(This.ItemIndex).Selected Then
          FullPathName = This.Item(This.ItemIndex).Text & "\"
          If FullPathName <> This.CurrentDir Then
            'Update the internal (Private) path variable so that it can be
            'retrieved externally using the (Public) Directory function above
            This.CurrentDir = FullPathName
          End If
        End If
      End If
      'Add any extra coding you might need... here
      'But remember OnKeyUp call this subroutine too
      '(meaning... avoid reputition)

      FullPathName = ""
    End Sub

    'If you have a need to do more than just retrieve the
    'path name in an OnKeyUp event... add your code to this subroutine
    'Do not create an external subroutine that never calls this subroutine
    '(if you do... you won't get the current directory name)
    Sub FolderTree_OnKeyUp(ByVal key As Byte, ByVal shift As Long)
      'Update the internal (Private) path variable so that it can be
      'retrieved externally using the (Public) Directory function above
      This.FolderTree_OnClick
      'Add any extra coding you might need... here
    End Sub

    Sub Initialize
      Dim I           As Long
      Dim RetVal      As Long
      Dim OldStyle    As Long
      Dim NewStyle    As Long
      Dim DriveLetter As String

      I           = 0
      RetVal      = 0
      OldStyle    = 0
      NewStyle    = 0
      DriveLetter = ""

      'Change the ToolTip style so it's does not show
      '(that lingering ToolTip is like sooooo annoying)
      OldStyle = GetWindowLong(This.Handle, GWL_STYLE)
      NewStyle = OldStyle OR TVS_NOTOOLTIPS
      SetWindowLong(This.Handle, GWL_STYLE, NewStyle)

      'Call the API function to get all "currently" available drives
      'The return value needs to be "bit" checked to determine
      'which ones are available and which ones are not
      '(a LONG has 32 bits but drive letters can only be... A..Z)
      RetVal = GetLogicalDrives()
      For I = 0 To 25
        If (RetVal And 2 ^ I) <> 0 Then
          'Using a starting point of 65 will provide Capital letters
          DriveLetter = Chr$(65 + I) & ":" 'Remember... NO "\" !
          'Add the drive letter to the QTreeView
          This.AddItems(DriveLetter)
          'Call the subroutine that'll add an icon
          'to the item (that was just added)
          'that's appropriate for the drive letter
          This.SetIcon(This.ItemCount-1, DriveLetter)
          'Call the subroutine that'll determine
          'whether or not to display a "+" button
          'next to the item that was just added
          This.GetFakeSubFolders(This.ItemCount-1, DriveLetter)
        End If
      Next I

      'Set the initial directory based on the
      'value of the (Public) InitialDir variable
      This.SetInitialDir(This.InitialDir)

      I           = 0
      RetVal      = 0
      OldStyle    = 0
      NewStyle    = 0
      DriveLetter = ""
    End Sub

    Constructor
      '** Start of items that should NOT be handled (or altered) externally **
      'Set the size of the icons to be added
      'If you decide to use larger icons update both of these
      IcoImageList.Width  = 16
      IcoImageList.Height = 16
      'Add the resource icons (from above) to the ImageList in the proper order
      IcoImageList.AddICOhandle(ICO1)
      IcoImageList.AddICOhandle(ICO2)
      IcoImageList.AddICOhandle(ICO3)
      IcoImageList.AddICOhandle(ICO4)
      IcoImageList.AddICOhandle(ICO5)
      IcoImageList.AddICOhandle(ICO6)
      'The purpose of this $Include file is for displaying ONLY
      ReadOnly = True
      'Set BOTH of these properties to the same ImageList
      'Create a whole new ImageList if you want StateImages
      'to appear when a QTreeView item is selected
      Images      = This.IcoImageList
      StateImages = This.IcoImageList
      'Handle these events internally not externally
      'Handling them externally means this $Include file will never work!
      OnExpanding  = This.FolderTree_OnExpanding
      OnCollapsing = This.FolderTree_OnCollapsing
      'There's no need to have the QFileListBox visible because
      'it's just being used to retrieve folder names
      'There's not even a need to set it's size or location
      DirList.Visible = False
      'Pre-set the variable that prevents OnExpanding from occurring 2x in a row
      ExpandedIndex = -1
      'Need to make sure CurrentDir a set to nothing before calling the SetInitialDir subroutine
      CurrentDir = ""
      '** End of items that should NOT be handled (or altered) externally **

      '** Start of items that CAN be altered externally **
      'Default to NOT showing every kind of folder
      SafeMode = True
      'Default to the initial directory to where the executable resides
      InitialDir = Command$(0) - Application.ExeName
      'Set some basic defaults (just in case)
      Left         = 0
      Top          = 0
      Width        = 320
      Height       = 240
      BorderStyle  = bsNone
      Font.Name    = "Arial"
      Font.Size    = 9
      '** End of items that CAN be altered externally **
    End Constructor
End Type
